/**
 * \file snakeGame.cpp
 * \brief Implements the game engine
 * \see snakeGame.h
 *
 * \date Nov 5, 2010
 * \author Sergey Levi <sergeyle@gmail.com>
 */

#include <ncurses.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>

#include "snakeGame.h"
#include "snake.h"

SnakeGame::SnakeGame(char food_symbol) :
    mScore (0),
    mFieldTop(1), mFieldLeft(1),
    mFieldHeight(LINES - 2), mFieldWidth(COLS - 2),
    mPlayWindow(newwin(mFieldHeight, mFieldWidth, 1, 1)),
    mSnake (Snake(Point(mFieldHeight / 2, mFieldWidth / 2), 4, mPlayWindow)),
    mFoodLocation(Point(mFieldTop + 1, mFieldWidth + 1)),
    mFoodSymbol(food_symbol)
{
    drawBoard();
    printTitle();
    displayMessageModal("ncurses Snake. Press any key to start. Press 'Q' to quit.");
    mSnake.drawSnake();
    placeFood();
    mvwaddch(mPlayWindow, mFoodLocation.row, mFoodLocation.col, mFoodSymbol);
    refresh();
}

void SnakeGame::drawBoard()
{
    box(mPlayWindow, 0, 0);
    wrefresh(mPlayWindow);
}

void SnakeGame::printTitle()
{
    const int score_strlen = strlen(FIELD_TITLE) + 8;
    mvprintw(0, (COLS - score_strlen) / 2, FIELD_TITLE "%4d", mScore);
    refresh();
}

void SnakeGame::gameLoop()
{
    int input;
    Direction dir = DirectionRight;
    while ((input = getch()) != 'q')
    {
        switch (input)
        {
            case KEY_UP:
                dir = DirectionUp;
                break;
            case KEY_RIGHT:
                dir = DirectionRight;
                break;
            case KEY_DOWN:
                dir = DirectionDown;
                break;
            case KEY_LEFT:
                dir = DirectionLeft;
                break;
        }

        wclear(mPlayWindow);
        drawBoard();
        mSnake.moveSnake(dir);
        
        if (!isInside(mSnake.getHead()) || mSnake.hasCollided())
        {
            break;
        }
        
        Point snake_head = mSnake.getHead();
        if (mFoodLocation == mSnake.getHead())
        {
            mSnake.eat();
            placeFood();
            mScore += 10;
        }

       
        
        mvwaddch(mPlayWindow, mFoodLocation.row, mFoodLocation.col, mFoodSymbol);
        mSnake.drawSnake();
        wrefresh(mPlayWindow);
        printTitle();
        refresh();
        usleep(100000);
    }

    displayMessageModal("Game over!");
    refresh();
}

void SnakeGame::placeFood()
{
    mFoodLocation.row = mFieldTop + 1 + rand() % (mFieldHeight - 1);
    mFoodLocation.col = mFieldLeft + 1 + rand() % (mFieldWidth - 1);
}

SnakeGame::~SnakeGame()
{
    delwin(mPlayWindow);
}

void SnakeGame::displayMessageModal(const char *msg)
{
    int msg_len = strlen(msg);
    WINDOW* msg_win = newwin(5, msg_len + 5,
                             (mFieldHeight - 5) / 2, (mFieldWidth - msg_len + 5) / 2);
    box(msg_win, 0, 0);
    mvwprintw(msg_win, 2, 2, msg);
    wrefresh(msg_win);
    timeout(-1);
    getch();
    timeout(0);
    delwin(msg_win);
}

bool SnakeGame::isInside(Point p)
{
    return (p.row > mFieldTop) && (p.row < mFieldTop + mFieldHeight - 2) &&
           (p.col > mFieldLeft) && (p.col < mFieldLeft + mFieldWidth - 2);
}

